package oliot.aika.koostaminen;

import oliot.aika.metodi.Aika;

/**
 * Luokan laajentaminen koostamalla
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 7.7.2011
 * @example
 * <pre name="test">
 * AikaSek a1 = new AikaSek(14,55,45);
 * a1.toString() === "14:55:45";
 * a1.lisaa(3,30); a1.toString() === "14:59:15";
 * </pre>
 */
public class AikaSek {

    private Aika hm = new Aika();
    private int s = 0;


    /**
     * Alustaa ajan sekuntien tarkkuudella
     * @param h tunnit
     * @param m minuutit
     * @param s sekunnit
     * @example
     * <pre name="test">
     * new AikaSek(11,12,13).toString() === "11:12:13"; 
     * </pre>
     */
    public AikaSek(int h, int m, int s) { // Muodostaja
        aseta(h, m, s);
    }


    /**
     * Alustaa ajan minuuttien tarkkuudella
     * @param h tunnit
     * @param m minuutit
     */
    public AikaSek(int h, int m) {
        aseta(h, m, 0);
    }


    /**
     * alustaa ajan tuntien tarkkuudella
     * @param h tunnit
     */
    public AikaSek(int h) {
        aseta(h, 0, 0);
    }


    /**
     * Alustataan aika 0:0:0
     */
    public AikaSek() {
        //
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param h asetettavat tunnit
     * @param m asetettavat minuutit
     * @param is asetettavat sekunnit
     * @return yli- tai alijneet vuorokaudet
     * @example
     * <pre name="test">
     * AikaSek aika = new AikaSek(1,2,3);
     * aika.toString() === "01:02:03";
     * aika.aseta(-1,32,42) === -1;
     * aika.toString() === "23:32:42";
     * </pre>
     */
    public final int aseta(int h, int m, int is) {
        int ts = is;
        int tm = m;
        tm += ts / 60; // liiat sekunnit minuutteihin
        ts %= 60; // sekunnit vlille -59 - 59
        if (ts < 0) {
            ts += 60;
            tm--;
        }
        int vrk = hm.aseta(h, tm);
        this.s = ts;
        return vrk;
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param h asetettavat tunnit
     * @param m asetettavat minuutit
     * @return yli- tai alijneet vuorokaudet
     */
    public final int aseta(int h, int m) {
        return aseta(h, m, 0);
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param h asetettavat tunnit
     * @return yli- tai alijneet vuorokaudet
     */
    public final int aseta(int h) {
        return aseta(h, 0);
    }


    /**
     * Lis aikaa sekuntien tarkkuudella
     * @param lisaMin listtv minuuttimr
     * @param lisaSek listtv sekunttimr
     * @example
     * <pre name="test">
     * AikaSek a1 = new AikaSek(11,12,13);
     * a1.lisaa(55,7); a1.toString() === "12:07:20";
     * </pre>
     */
    public void lisaa(int lisaMin, int lisaSek) {
        aseta(hm.getH(), hm.getM() + lisaMin, this.s + lisaSek);
    }


    /**
     * Lis aikaa minuuttien tarkkuudella
     * @param lisaMin listtv minuuttimr
     * @example
     * <pre name="test">
     * AikaSek a1 = new AikaSek(11,12,13);
     * a1.lisaa(55); a1.toString() === "12:07:13";
     * </pre>
     */
    public void lisaa(int lisaMin) {
        lisaa(lisaMin, 0);
    }


    /**
     * @return aika merkkijonona muodossa 12:05:03
     * @example
     * <pre name="test">
     * new AikaSek(11,12,13).toString() === "11:12:13";
     * </pre>
     */
    @Override
    public String toString() {
        return hm.toString() + String.format(":%02d", s);
    }


    /**
     * Pohjelma jolla demonstroidaan kmpel tapaa toteuttaa
     * polymorfismi koostetulla luokalla.
     * @param args ei kytss
     */
    public static void main(String[] args) {

        Aika a1 = new Aika();
        Aika a2 = new Aika(13);
        Aika a3 = new Aika(14, 175);
        AikaSek a4 = new AikaSek(14, 55, 45);

        Object ajat[] = new Object[5];
        ajat[0] = a1;
        ajat[1] = a2;
        ajat[2] = a3;
        ajat[3] = a4;
        ajat[4] = new AikaSek(23, 59, 59);

        for (int i = 0; i < ajat.length; i++) {
            if (ajat[i] instanceof Aika) {
                Aika aika = (Aika) ajat[i]; // pakotettu tyypin muunnos
                System.out.print(aika + " +" + i + " => ");
                aika.lisaa(i);
                System.out.println(aika);
            }
            if (ajat[i] instanceof AikaSek) {
                AikaSek aika = (AikaSek) ajat[i]; // pakotettu tyypin muunnos
                System.out.print(aika + " +" + i + " => ");
                aika.lisaa(i);
                System.out.println(aika);
            }
        }
    }
}
